"Main interface for mgh Client"
from __future__ import annotations

from datetime import datetime
from typing import Any, Dict, List
from typing_extensions import Literal, overload
from botocore.client import BaseClient
from botocore.exceptions import ClientError as Boto3ClientError
from botocore.paginate import Paginator as Boto3Paginator

# pylint: disable=import-self
import mypy_boto3_mgh.client as client_scope

# pylint: disable=import-self
import mypy_boto3_mgh.paginator as paginator_scope
from mypy_boto3_mgh.type_defs import (
    ClientAssociateCreatedArtifactCreatedArtifactTypeDef,
    ClientAssociateDiscoveredResourceDiscoveredResourceTypeDef,
    ClientDescribeApplicationStateResponseTypeDef,
    ClientDescribeMigrationTaskResponseTypeDef,
    ClientListCreatedArtifactsResponseTypeDef,
    ClientListDiscoveredResourcesResponseTypeDef,
    ClientListMigrationTasksResponseTypeDef,
    ClientListProgressUpdateStreamsResponseTypeDef,
    ClientNotifyMigrationTaskStateTaskTypeDef,
    ClientPutResourceAttributesResourceAttributeListTypeDef,
)


__all__ = ("Client",)


class Client(BaseClient):
    exceptions: client_scope.Exceptions

    # pylint: disable=arguments-differ,redefined-outer-name,redefined-builtin
    def associate_created_artifact(
        self,
        ProgressUpdateStream: str,
        MigrationTaskName: str,
        CreatedArtifact: ClientAssociateCreatedArtifactCreatedArtifactTypeDef,
        DryRun: bool = None,
    ) -> Dict[str, Any]:
        """
        Associates a created artifact of an AWS cloud resource, the target receiving the migration,
        with the migration task performed by a migration tool. This API has the following traits:

        * Migration tools can call the ``AssociateCreatedArtifact`` operation to indicate which AWS
        artifact is associated with a migration task.

        * The created artifact name must be provided in ARN (Amazon Resource Name) format which will
        contain information about type and region; for example:
        ``arn:aws:ec2:us-east-1:488216288981:image/ami-6d0ba87b`` .

        * Examples of the AWS resource behind the created artifact are, AMI's, EC2 instance, or DMS
        endpoint, etc.

        See also: `AWS API Documentation
        <https://docs.aws.amazon.com/goto/WebAPI/AWSMigrationHub-2017-05-31/AssociateCreatedArtifact>`_

        **Request Syntax**
        ::

          response = client.associate_created_artifact(
              ProgressUpdateStream='string',
              MigrationTaskName='string',
              CreatedArtifact={
                  'Name': 'string',
                  'Description': 'string'
              },
              DryRun=True|False
          )
        :type ProgressUpdateStream: string
        :param ProgressUpdateStream: **[REQUIRED]**

          The name of the ProgressUpdateStream.

        :type MigrationTaskName: string
        :param MigrationTaskName: **[REQUIRED]**

          Unique identifier that references the migration task. *Do not store personal data in this
          field.*

        :type CreatedArtifact: dict
        :param CreatedArtifact: **[REQUIRED]**

          An ARN of the AWS resource related to the migration (e.g., AMI, EC2 instance, RDS
          instance, etc.)

          - **Name** *(string) --* **[REQUIRED]**

            An ARN that uniquely identifies the result of a migration task.

          - **Description** *(string) --*

            A description that can be free-form text to record additional detail about the artifact
            for clarity or for later reference.

        :type DryRun: boolean
        :param DryRun:

          Optional boolean flag to indicate whether any effect should take place. Used to test if
          the caller has permission to make the call.

        :rtype: dict
        :returns:

          **Response Syntax**

          ::

            {}
          **Response Structure**

          - *(dict) --*
        """

    # pylint: disable=arguments-differ,redefined-outer-name,redefined-builtin
    def associate_discovered_resource(
        self,
        ProgressUpdateStream: str,
        MigrationTaskName: str,
        DiscoveredResource: ClientAssociateDiscoveredResourceDiscoveredResourceTypeDef,
        DryRun: bool = None,
    ) -> Dict[str, Any]:
        """
        Associates a discovered resource ID from Application Discovery Service with a migration
        task.

        See also: `AWS API Documentation
        <https://docs.aws.amazon.com/goto/WebAPI/AWSMigrationHub-2017-05-31/AssociateDiscoveredResource>`_

        **Request Syntax**
        ::

          response = client.associate_discovered_resource(
              ProgressUpdateStream='string',
              MigrationTaskName='string',
              DiscoveredResource={
                  'ConfigurationId': 'string',
                  'Description': 'string'
              },
              DryRun=True|False
          )
        :type ProgressUpdateStream: string
        :param ProgressUpdateStream: **[REQUIRED]**

          The name of the ProgressUpdateStream.

        :type MigrationTaskName: string
        :param MigrationTaskName: **[REQUIRED]**

          The identifier given to the MigrationTask. *Do not store personal data in this field.*

        :type DiscoveredResource: dict
        :param DiscoveredResource: **[REQUIRED]**

          Object representing a Resource.

          - **ConfigurationId** *(string) --* **[REQUIRED]**

            The configurationId in Application Discovery Service that uniquely identifies the
            on-premise resource.

          - **Description** *(string) --*

            A description that can be free-form text to record additional detail about the
            discovered resource for clarity or later reference.

        :type DryRun: boolean
        :param DryRun:

          Optional boolean flag to indicate whether any effect should take place. Used to test if
          the caller has permission to make the call.

        :rtype: dict
        :returns:

          **Response Syntax**

          ::

            {}
          **Response Structure**

          - *(dict) --*
        """

    # pylint: disable=arguments-differ,redefined-outer-name,redefined-builtin
    def can_paginate(self, operation_name: str) -> None:
        """
        Check if an operation can be paginated.

        :type operation_name: string
        :param operation_name: The operation name.  This is the same name
            as the method name on the client.  For example, if the
            method name is ``create_foo``, and you'd normally invoke the
            operation as ``client.create_foo(**kwargs)``, if the
            ``create_foo`` operation can be paginated, you can use the
            call ``client.get_paginator("create_foo")``.

        :return: ``True`` if the operation can be paginated,
            ``False`` otherwise.
        """

    # pylint: disable=arguments-differ,redefined-outer-name,redefined-builtin
    def create_progress_update_stream(
        self, ProgressUpdateStreamName: str, DryRun: bool = None
    ) -> Dict[str, Any]:
        """
        Creates a progress update stream which is an AWS resource used for access control as well as
        a namespace for migration task names that is implicitly linked to your AWS account. It must
        uniquely identify the migration tool as it is used for all updates made by the tool;
        however, it does not need to be unique for each AWS account because it is scoped to the AWS
        account.

        See also: `AWS API Documentation
        <https://docs.aws.amazon.com/goto/WebAPI/AWSMigrationHub-2017-05-31/CreateProgressUpdateStream>`_

        **Request Syntax**
        ::

          response = client.create_progress_update_stream(
              ProgressUpdateStreamName='string',
              DryRun=True|False
          )
        :type ProgressUpdateStreamName: string
        :param ProgressUpdateStreamName: **[REQUIRED]**

          The name of the ProgressUpdateStream. *Do not store personal data in this field.*

        :type DryRun: boolean
        :param DryRun:

          Optional boolean flag to indicate whether any effect should take place. Used to test if
          the caller has permission to make the call.

        :rtype: dict
        :returns:

          **Response Syntax**

          ::

            {}
          **Response Structure**

          - *(dict) --*
        """

    # pylint: disable=arguments-differ,redefined-outer-name,redefined-builtin
    def delete_progress_update_stream(
        self, ProgressUpdateStreamName: str, DryRun: bool = None
    ) -> Dict[str, Any]:
        """
        Deletes a progress update stream, including all of its tasks, which was previously created
        as an AWS resource used for access control. This API has the following traits:

        * The only parameter needed for ``DeleteProgressUpdateStream`` is the stream name (same as a
        ``CreateProgressUpdateStream`` call).

        * The call will return, and a background process will asynchronously delete the stream and
        all of its resources (tasks, associated resources, resource attributes, created artifacts).

        * If the stream takes time to be deleted, it might still show up on a
        ``ListProgressUpdateStreams`` call.

        * ``CreateProgressUpdateStream`` , ``ImportMigrationTask`` , ``NotifyMigrationTaskState`` ,
        and all Associate[*] APIs related to the tasks belonging to the stream will throw
        "InvalidInputException" if the stream of the same name is in the process of being deleted.

        * Once the stream and all of its resources are deleted, ``CreateProgressUpdateStream`` for a
        stream of the same name will succeed, and that stream will be an entirely new logical
        resource (without any resources associated with the old stream).

        See also: `AWS API Documentation
        <https://docs.aws.amazon.com/goto/WebAPI/AWSMigrationHub-2017-05-31/DeleteProgressUpdateStream>`_

        **Request Syntax**
        ::

          response = client.delete_progress_update_stream(
              ProgressUpdateStreamName='string',
              DryRun=True|False
          )
        :type ProgressUpdateStreamName: string
        :param ProgressUpdateStreamName: **[REQUIRED]**

          The name of the ProgressUpdateStream. *Do not store personal data in this field.*

        :type DryRun: boolean
        :param DryRun:

          Optional boolean flag to indicate whether any effect should take place. Used to test if
          the caller has permission to make the call.

        :rtype: dict
        :returns:

          **Response Syntax**

          ::

            {}
          **Response Structure**

          - *(dict) --*
        """

    # pylint: disable=arguments-differ,redefined-outer-name,redefined-builtin
    def describe_application_state(
        self, ApplicationId: str
    ) -> ClientDescribeApplicationStateResponseTypeDef:
        """
        Gets the migration status of an application.

        See also: `AWS API Documentation
        <https://docs.aws.amazon.com/goto/WebAPI/AWSMigrationHub-2017-05-31/DescribeApplicationState>`_

        **Request Syntax**
        ::

          response = client.describe_application_state(
              ApplicationId='string'
          )
        :type ApplicationId: string
        :param ApplicationId: **[REQUIRED]**

          The configurationId in Application Discovery Service that uniquely identifies the grouped
          application.

        :rtype: dict
        :returns:

          **Response Syntax**

          ::

            {
                'ApplicationStatus': 'NOT_STARTED'|'IN_PROGRESS'|'COMPLETED',
                'LastUpdatedTime': datetime(2015, 1, 1)
            }
          **Response Structure**

          - *(dict) --*

            - **ApplicationStatus** *(string) --*

              Status of the application - Not Started, In-Progress, Complete.

            - **LastUpdatedTime** *(datetime) --*

              The timestamp when the application status was last updated.

        """

    # pylint: disable=arguments-differ,redefined-outer-name,redefined-builtin
    def describe_migration_task(
        self, ProgressUpdateStream: str, MigrationTaskName: str
    ) -> ClientDescribeMigrationTaskResponseTypeDef:
        """
        Retrieves a list of all attributes associated with a specific migration task.

        See also: `AWS API Documentation
        <https://docs.aws.amazon.com/goto/WebAPI/AWSMigrationHub-2017-05-31/DescribeMigrationTask>`_

        **Request Syntax**
        ::

          response = client.describe_migration_task(
              ProgressUpdateStream='string',
              MigrationTaskName='string'
          )
        :type ProgressUpdateStream: string
        :param ProgressUpdateStream: **[REQUIRED]**

          The name of the ProgressUpdateStream.

        :type MigrationTaskName: string
        :param MigrationTaskName: **[REQUIRED]**

          The identifier given to the MigrationTask. *Do not store personal data in this field.*

        :rtype: dict
        :returns:

          **Response Syntax**

          ::

            {
                'MigrationTask': {
                    'ProgressUpdateStream': 'string',
                    'MigrationTaskName': 'string',
                    'Task': {
                        'Status': 'NOT_STARTED'|'IN_PROGRESS'|'FAILED'|'COMPLETED',
                        'StatusDetail': 'string',
                        'ProgressPercent': 123
                    },
                    'UpdateDateTime': datetime(2015, 1, 1),
                    'ResourceAttributeList': [
                        {
                            'Type':
                            'IPV4_ADDRESS'|'IPV6_ADDRESS'|'MAC_ADDRESS'|'FQDN'
                            |'VM_MANAGER_ID'|'VM_MANAGED_OBJECT_REFERENCE'
                            |'VM_NAME'|'VM_PATH'|'BIOS_ID'
                            |'MOTHERBOARD_SERIAL_NUMBER',
                            'Value': 'string'
                        },
                    ]
                }
            }
          **Response Structure**

          - *(dict) --*

            - **MigrationTask** *(dict) --*

              Object encapsulating information about the migration task.

              - **ProgressUpdateStream** *(string) --*

                A name that identifies the vendor of the migration tool being used.

              - **MigrationTaskName** *(string) --*

                Unique identifier that references the migration task. *Do not store personal data in
                this field.*

              - **Task** *(dict) --*

                Task object encapsulating task information.

                - **Status** *(string) --*

                  Status of the task - Not Started, In-Progress, Complete.

                - **StatusDetail** *(string) --*

                  Details of task status as notified by a migration tool. A tool might use this
                  field to provide clarifying information about the status that is unique to that
                  tool or that explains an error state.

                - **ProgressPercent** *(integer) --*

                  Indication of the percentage completion of the task.

              - **UpdateDateTime** *(datetime) --*

                The timestamp when the task was gathered.

              - **ResourceAttributeList** *(list) --*

                Information about the resource that is being migrated. This data will be used to map
                the task to a resource in the Application Discovery Service repository.

                - *(dict) --*

                  Attribute associated with a resource.

                  Note the corresponding format required per type listed below:

                    IPV4

                   ``x.x.x.x``

                   *where x is an integer in the range [0,255]*

                    IPV6

                   ``y : y : y : y : y : y : y : y``

                   *where y is a hexadecimal between 0 and FFFF. [0, FFFF]*

                    MAC_ADDRESS

                   ``^([0-9A-Fa-f]{2}[:-]){5}([0-9A-Fa-f]{2})$``

                    FQDN

                   ``^[^<>{}\\\\/?,=\\p{Cntrl}]{1,256}$``

                  - **Type** *(string) --*

                    Type of resource.

                  - **Value** *(string) --*

                    Value of the resource type.

        """

    # pylint: disable=arguments-differ,redefined-outer-name,redefined-builtin
    def disassociate_created_artifact(
        self,
        ProgressUpdateStream: str,
        MigrationTaskName: str,
        CreatedArtifactName: str,
        DryRun: bool = None,
    ) -> Dict[str, Any]:
        """
        Disassociates a created artifact of an AWS resource with a migration task performed by a
        migration tool that was previously associated. This API has the following traits:

        * A migration user can call the ``DisassociateCreatedArtifacts`` operation to disassociate a
        created AWS Artifact from a migration task.

        * The created artifact name must be provided in ARN (Amazon Resource Name) format which will
        contain information about type and region; for example:
        ``arn:aws:ec2:us-east-1:488216288981:image/ami-6d0ba87b`` .

        * Examples of the AWS resource behind the created artifact are, AMI's, EC2 instance, or RDS
        instance, etc.

        See also: `AWS API Documentation
        <https://docs.aws.amazon.com/goto/WebAPI/AWSMigrationHub-2017-05-31/DisassociateCreatedArtifact>`_

        **Request Syntax**
        ::

          response = client.disassociate_created_artifact(
              ProgressUpdateStream='string',
              MigrationTaskName='string',
              CreatedArtifactName='string',
              DryRun=True|False
          )
        :type ProgressUpdateStream: string
        :param ProgressUpdateStream: **[REQUIRED]**

          The name of the ProgressUpdateStream.

        :type MigrationTaskName: string
        :param MigrationTaskName: **[REQUIRED]**

          Unique identifier that references the migration task to be disassociated with the
          artifact. *Do not store personal data in this field.*

        :type CreatedArtifactName: string
        :param CreatedArtifactName: **[REQUIRED]**

          An ARN of the AWS resource related to the migration (e.g., AMI, EC2 instance, RDS
          instance, etc.)

        :type DryRun: boolean
        :param DryRun:

          Optional boolean flag to indicate whether any effect should take place. Used to test if
          the caller has permission to make the call.

        :rtype: dict
        :returns:

          **Response Syntax**

          ::

            {}
          **Response Structure**

          - *(dict) --*
        """

    # pylint: disable=arguments-differ,redefined-outer-name,redefined-builtin
    def disassociate_discovered_resource(
        self,
        ProgressUpdateStream: str,
        MigrationTaskName: str,
        ConfigurationId: str,
        DryRun: bool = None,
    ) -> Dict[str, Any]:
        """
        Disassociate an Application Discovery Service discovered resource from a migration task.

        See also: `AWS API Documentation
        <https://docs.aws.amazon.com/goto/WebAPI/AWSMigrationHub-2017-05-31/DisassociateDiscoveredResource>`_

        **Request Syntax**
        ::

          response = client.disassociate_discovered_resource(
              ProgressUpdateStream='string',
              MigrationTaskName='string',
              ConfigurationId='string',
              DryRun=True|False
          )
        :type ProgressUpdateStream: string
        :param ProgressUpdateStream: **[REQUIRED]**

          The name of the ProgressUpdateStream.

        :type MigrationTaskName: string
        :param MigrationTaskName: **[REQUIRED]**

          The identifier given to the MigrationTask. *Do not store personal data in this field.*

        :type ConfigurationId: string
        :param ConfigurationId: **[REQUIRED]**

          ConfigurationId of the Application Discovery Service resource to be disassociated.

        :type DryRun: boolean
        :param DryRun:

          Optional boolean flag to indicate whether any effect should take place. Used to test if
          the caller has permission to make the call.

        :rtype: dict
        :returns:

          **Response Syntax**

          ::

            {}
          **Response Structure**

          - *(dict) --*
        """

    # pylint: disable=arguments-differ,redefined-outer-name,redefined-builtin
    def generate_presigned_url(
        self, ClientMethod: str, Params: Dict = None, ExpiresIn: int = 3600, HttpMethod: str = None
    ) -> None:
        """
        Generate a presigned url given a client, its method, and arguments

        :type ClientMethod: string
        :param ClientMethod: The client method to presign for

        :type Params: dict
        :param Params: The parameters normally passed to
            ``ClientMethod``.

        :type ExpiresIn: int
        :param ExpiresIn: The number of seconds the presigned url is valid
            for. By default it expires in an hour (3600 seconds)

        :type HttpMethod: string
        :param HttpMethod: The http method to use on the generated url. By
            default, the http method is whatever is used in the method's model.

        :returns: The presigned url
        """

    # pylint: disable=arguments-differ,redefined-outer-name,redefined-builtin
    def import_migration_task(
        self, ProgressUpdateStream: str, MigrationTaskName: str, DryRun: bool = None
    ) -> Dict[str, Any]:
        """
        Registers a new migration task which represents a server, database, etc., being migrated to
        AWS by a migration tool.

        This API is a prerequisite to calling the ``NotifyMigrationTaskState`` API as the migration
        tool must first register the migration task with Migration Hub.

        See also: `AWS API Documentation
        <https://docs.aws.amazon.com/goto/WebAPI/AWSMigrationHub-2017-05-31/ImportMigrationTask>`_

        **Request Syntax**
        ::

          response = client.import_migration_task(
              ProgressUpdateStream='string',
              MigrationTaskName='string',
              DryRun=True|False
          )
        :type ProgressUpdateStream: string
        :param ProgressUpdateStream: **[REQUIRED]**

          The name of the ProgressUpdateStream. >

        :type MigrationTaskName: string
        :param MigrationTaskName: **[REQUIRED]**

          Unique identifier that references the migration task. *Do not store personal data in this
          field.*

        :type DryRun: boolean
        :param DryRun:

          Optional boolean flag to indicate whether any effect should take place. Used to test if
          the caller has permission to make the call.

        :rtype: dict
        :returns:

          **Response Syntax**

          ::

            {}
          **Response Structure**

          - *(dict) --*
        """

    # pylint: disable=arguments-differ,redefined-outer-name,redefined-builtin
    def list_created_artifacts(
        self,
        ProgressUpdateStream: str,
        MigrationTaskName: str,
        NextToken: str = None,
        MaxResults: int = None,
    ) -> ClientListCreatedArtifactsResponseTypeDef:
        """
        Lists the created artifacts attached to a given migration task in an update stream. This API
        has the following traits:

        * Gets the list of the created artifacts while migration is taking place.

        * Shows the artifacts created by the migration tool that was associated by the
        ``AssociateCreatedArtifact`` API.

        * Lists created artifacts in a paginated interface.

        See also: `AWS API Documentation
        <https://docs.aws.amazon.com/goto/WebAPI/AWSMigrationHub-2017-05-31/ListCreatedArtifacts>`_

        **Request Syntax**
        ::

          response = client.list_created_artifacts(
              ProgressUpdateStream='string',
              MigrationTaskName='string',
              NextToken='string',
              MaxResults=123
          )
        :type ProgressUpdateStream: string
        :param ProgressUpdateStream: **[REQUIRED]**

          The name of the ProgressUpdateStream.

        :type MigrationTaskName: string
        :param MigrationTaskName: **[REQUIRED]**

          Unique identifier that references the migration task. *Do not store personal data in this
          field.*

        :type NextToken: string
        :param NextToken:

          If a ``NextToken`` was returned by a previous call, there are more results available. To
          retrieve the next page of results, make the call again using the returned token in
          ``NextToken`` .

        :type MaxResults: integer
        :param MaxResults:

          Maximum number of results to be returned per page.

        :rtype: dict
        :returns:

          **Response Syntax**

          ::

            {
                'NextToken': 'string',
                'CreatedArtifactList': [
                    {
                        'Name': 'string',
                        'Description': 'string'
                    },
                ]
            }
          **Response Structure**

          - *(dict) --*

            - **NextToken** *(string) --*

              If there are more created artifacts than the max result, return the next token to be
              passed to the next call as a bookmark of where to start from.

            - **CreatedArtifactList** *(list) --*

              List of created artifacts up to the maximum number of results specified in the
              request.

              - *(dict) --*

                An ARN of the AWS cloud resource target receiving the migration (e.g., AMI, EC2
                instance, RDS instance, etc.).

                - **Name** *(string) --*

                  An ARN that uniquely identifies the result of a migration task.

                - **Description** *(string) --*

                  A description that can be free-form text to record additional detail about the
                  artifact for clarity or for later reference.

        """

    # pylint: disable=arguments-differ,redefined-outer-name,redefined-builtin
    def list_discovered_resources(
        self,
        ProgressUpdateStream: str,
        MigrationTaskName: str,
        NextToken: str = None,
        MaxResults: int = None,
    ) -> ClientListDiscoveredResourcesResponseTypeDef:
        """
        Lists discovered resources associated with the given ``MigrationTask`` .

        See also: `AWS API Documentation
        <https://docs.aws.amazon.com/goto/WebAPI/AWSMigrationHub-2017-05-31/ListDiscoveredResources>`_

        **Request Syntax**
        ::

          response = client.list_discovered_resources(
              ProgressUpdateStream='string',
              MigrationTaskName='string',
              NextToken='string',
              MaxResults=123
          )
        :type ProgressUpdateStream: string
        :param ProgressUpdateStream: **[REQUIRED]**

          The name of the ProgressUpdateStream.

        :type MigrationTaskName: string
        :param MigrationTaskName: **[REQUIRED]**

          The name of the MigrationTask. *Do not store personal data in this field.*

        :type NextToken: string
        :param NextToken:

          If a ``NextToken`` was returned by a previous call, there are more results available. To
          retrieve the next page of results, make the call again using the returned token in
          ``NextToken`` .

        :type MaxResults: integer
        :param MaxResults:

          The maximum number of results returned per page.

        :rtype: dict
        :returns:

          **Response Syntax**

          ::

            {
                'NextToken': 'string',
                'DiscoveredResourceList': [
                    {
                        'ConfigurationId': 'string',
                        'Description': 'string'
                    },
                ]
            }
          **Response Structure**

          - *(dict) --*

            - **NextToken** *(string) --*

              If there are more discovered resources than the max result, return the next token to
              be passed to the next call as a bookmark of where to start from.

            - **DiscoveredResourceList** *(list) --*

              Returned list of discovered resources associated with the given MigrationTask.

              - *(dict) --*

                Object representing the on-premises resource being migrated.

                - **ConfigurationId** *(string) --*

                  The configurationId in Application Discovery Service that uniquely identifies the
                  on-premise resource.

                - **Description** *(string) --*

                  A description that can be free-form text to record additional detail about the
                  discovered resource for clarity or later reference.

        """

    # pylint: disable=arguments-differ,redefined-outer-name,redefined-builtin
    def list_migration_tasks(
        self, NextToken: str = None, MaxResults: int = None, ResourceName: str = None
    ) -> ClientListMigrationTasksResponseTypeDef:
        """
        Lists all, or filtered by resource name, migration tasks associated with the user account
        making this call. This API has the following traits:

        * Can show a summary list of the most recent migration tasks.

        * Can show a summary list of migration tasks associated with a given discovered resource.

        * Lists migration tasks in a paginated interface.

        See also: `AWS API Documentation
        <https://docs.aws.amazon.com/goto/WebAPI/AWSMigrationHub-2017-05-31/ListMigrationTasks>`_

        **Request Syntax**
        ::

          response = client.list_migration_tasks(
              NextToken='string',
              MaxResults=123,
              ResourceName='string'
          )
        :type NextToken: string
        :param NextToken:

          If a ``NextToken`` was returned by a previous call, there are more results available. To
          retrieve the next page of results, make the call again using the returned token in
          ``NextToken`` .

        :type MaxResults: integer
        :param MaxResults:

          Value to specify how many results are returned per page.

        :type ResourceName: string
        :param ResourceName:

          Filter migration tasks by discovered resource name.

        :rtype: dict
        :returns:

          **Response Syntax**

          ::

            {
                'NextToken': 'string',
                'MigrationTaskSummaryList': [
                    {
                        'ProgressUpdateStream': 'string',
                        'MigrationTaskName': 'string',
                        'Status': 'NOT_STARTED'|'IN_PROGRESS'|'FAILED'|'COMPLETED',
                        'ProgressPercent': 123,
                        'StatusDetail': 'string',
                        'UpdateDateTime': datetime(2015, 1, 1)
                    },
                ]
            }
          **Response Structure**

          - *(dict) --*

            - **NextToken** *(string) --*

              If there are more migration tasks than the max result, return the next token to be
              passed to the next call as a bookmark of where to start from.

            - **MigrationTaskSummaryList** *(list) --*

              Lists the migration task's summary which includes: ``MigrationTaskName`` ,
              ``ProgressPercent`` , ``ProgressUpdateStream`` , ``Status`` , and the
              ``UpdateDateTime`` for each task.

              - *(dict) --*

                MigrationTaskSummary includes ``MigrationTaskName`` , ``ProgressPercent`` ,
                ``ProgressUpdateStream`` , ``Status`` , and ``UpdateDateTime`` for each task.

                - **ProgressUpdateStream** *(string) --*

                  An AWS resource used for access control. It should uniquely identify the migration
                  tool as it is used for all updates made by the tool.

                - **MigrationTaskName** *(string) --*

                  Unique identifier that references the migration task. *Do not store personal data
                  in this field.*

                - **Status** *(string) --*

                  Status of the task.

                - **ProgressPercent** *(integer) --*

                  Indication of the percentage completion of the task.

                - **StatusDetail** *(string) --*

                  Detail information of what is being done within the overall status state.

                - **UpdateDateTime** *(datetime) --*

                  The timestamp when the task was gathered.

        """

    # pylint: disable=arguments-differ,redefined-outer-name,redefined-builtin
    def list_progress_update_streams(
        self, NextToken: str = None, MaxResults: int = None
    ) -> ClientListProgressUpdateStreamsResponseTypeDef:
        """
        Lists progress update streams associated with the user account making this call.

        See also: `AWS API Documentation
        <https://docs.aws.amazon.com/goto/WebAPI/AWSMigrationHub-2017-05-31/ListProgressUpdateStreams>`_

        **Request Syntax**
        ::

          response = client.list_progress_update_streams(
              NextToken='string',
              MaxResults=123
          )
        :type NextToken: string
        :param NextToken:

          If a ``NextToken`` was returned by a previous call, there are more results available. To
          retrieve the next page of results, make the call again using the returned token in
          ``NextToken`` .

        :type MaxResults: integer
        :param MaxResults:

          Filter to limit the maximum number of results to list per page.

        :rtype: dict
        :returns:

          **Response Syntax**

          ::

            {
                'ProgressUpdateStreamSummaryList': [
                    {
                        'ProgressUpdateStreamName': 'string'
                    },
                ],
                'NextToken': 'string'
            }
          **Response Structure**

          - *(dict) --*

            - **ProgressUpdateStreamSummaryList** *(list) --*

              List of progress update streams up to the max number of results passed in the input.

              - *(dict) --*

                Summary of the AWS resource used for access control that is implicitly linked to
                your AWS account.

                - **ProgressUpdateStreamName** *(string) --*

                  The name of the ProgressUpdateStream. *Do not store personal data in this field.*

            - **NextToken** *(string) --*

              If there are more streams created than the max result, return the next token to be
              passed to the next call as a bookmark of where to start from.

        """

    # pylint: disable=arguments-differ,redefined-outer-name,redefined-builtin
    def notify_application_state(
        self, ApplicationId: str, Status: str, UpdateDateTime: datetime = None, DryRun: bool = None
    ) -> Dict[str, Any]:
        """
        Sets the migration state of an application. For a given application identified by the value
        passed to ``ApplicationId`` , its status is set or updated by passing one of three values to
        ``Status`` : ``NOT_STARTED | IN_PROGRESS | COMPLETED`` .

        See also: `AWS API Documentation
        <https://docs.aws.amazon.com/goto/WebAPI/AWSMigrationHub-2017-05-31/NotifyApplicationState>`_

        **Request Syntax**
        ::

          response = client.notify_application_state(
              ApplicationId='string',
              Status='NOT_STARTED'|'IN_PROGRESS'|'COMPLETED',
              UpdateDateTime=datetime(2015, 1, 1),
              DryRun=True|False
          )
        :type ApplicationId: string
        :param ApplicationId: **[REQUIRED]**

          The configurationId in Application Discovery Service that uniquely identifies the grouped
          application.

        :type Status: string
        :param Status: **[REQUIRED]**

          Status of the application - Not Started, In-Progress, Complete.

        :type UpdateDateTime: datetime
        :param UpdateDateTime:

          The timestamp when the application state changed.

        :type DryRun: boolean
        :param DryRun:

          Optional boolean flag to indicate whether any effect should take place. Used to test if
          the caller has permission to make the call.

        :rtype: dict
        :returns:

          **Response Syntax**

          ::

            {}
          **Response Structure**

          - *(dict) --*
        """

    # pylint: disable=arguments-differ,redefined-outer-name,redefined-builtin
    def notify_migration_task_state(
        self,
        ProgressUpdateStream: str,
        MigrationTaskName: str,
        Task: ClientNotifyMigrationTaskStateTaskTypeDef,
        UpdateDateTime: datetime,
        NextUpdateSeconds: int,
        DryRun: bool = None,
    ) -> Dict[str, Any]:
        """
        Notifies Migration Hub of the current status, progress, or other detail regarding a
        migration task. This API has the following traits:

        * Migration tools will call the ``NotifyMigrationTaskState`` API to share the latest
        progress and status.

        * ``MigrationTaskName`` is used for addressing updates to the correct target.

        * ``ProgressUpdateStream`` is used for access control and to provide a namespace for each
        migration tool.

        See also: `AWS API Documentation
        <https://docs.aws.amazon.com/goto/WebAPI/AWSMigrationHub-2017-05-31/NotifyMigrationTaskState>`_

        **Request Syntax**
        ::

          response = client.notify_migration_task_state(
              ProgressUpdateStream='string',
              MigrationTaskName='string',
              Task={
                  'Status': 'NOT_STARTED'|'IN_PROGRESS'|'FAILED'|'COMPLETED',
                  'StatusDetail': 'string',
                  'ProgressPercent': 123
              },
              UpdateDateTime=datetime(2015, 1, 1),
              NextUpdateSeconds=123,
              DryRun=True|False
          )
        :type ProgressUpdateStream: string
        :param ProgressUpdateStream: **[REQUIRED]**

          The name of the ProgressUpdateStream.

        :type MigrationTaskName: string
        :param MigrationTaskName: **[REQUIRED]**

          Unique identifier that references the migration task. *Do not store personal data in this
          field.*

        :type Task: dict
        :param Task: **[REQUIRED]**

          Information about the task's progress and status.

          - **Status** *(string) --* **[REQUIRED]**

            Status of the task - Not Started, In-Progress, Complete.

          - **StatusDetail** *(string) --*

            Details of task status as notified by a migration tool. A tool might use this field to
            provide clarifying information about the status that is unique to that tool or that
            explains an error state.

          - **ProgressPercent** *(integer) --*

            Indication of the percentage completion of the task.

        :type UpdateDateTime: datetime
        :param UpdateDateTime: **[REQUIRED]**

          The timestamp when the task was gathered.

        :type NextUpdateSeconds: integer
        :param NextUpdateSeconds: **[REQUIRED]**

          Number of seconds after the UpdateDateTime within which the Migration Hub can expect an
          update. If Migration Hub does not receive an update within the specified interval, then
          the migration task will be considered stale.

        :type DryRun: boolean
        :param DryRun:

          Optional boolean flag to indicate whether any effect should take place. Used to test if
          the caller has permission to make the call.

        :rtype: dict
        :returns:

          **Response Syntax**

          ::

            {}
          **Response Structure**

          - *(dict) --*
        """

    # pylint: disable=arguments-differ,redefined-outer-name,redefined-builtin
    def put_resource_attributes(
        self,
        ProgressUpdateStream: str,
        MigrationTaskName: str,
        ResourceAttributeList: List[ClientPutResourceAttributesResourceAttributeListTypeDef],
        DryRun: bool = None,
    ) -> Dict[str, Any]:
        """
        ..
        _https://docs.aws.amazon.com/migrationhub/latest/ug/API_PutResourceAttributes.html#migrationhub-PutResourceAttributes-request-ResourceAttributeList:
        https://docs.aws.amazon.com/migrationhub/latest/ug/API_PutResourceAttributes.html#migrationhub-PutResourceAttributes-request-ResourceAttributeList

        Provides identifying details of the resource being migrated so that it can be associated in
        the Application Discovery Service repository. This association occurs asynchronously after
        ``PutResourceAttributes`` returns.

        .. warning::

          * Keep in mind that subsequent calls to PutResourceAttributes will override previously
          stored attributes. For example, if it is first called with a MAC address, but later, it is
          desired to *add* an IP address, it will then be required to call it with *both* the IP and
          MAC addresses to prevent overriding the MAC address.

          * Note the instructions regarding the special use case of the ` ``ResourceAttributeList``
          https://docs.aws.amazon.com/migrationhub/latest/ug/API_PutResourceAttributes.html#migrationhub-PutResourceAttributes-request-ResourceAttributeList`__
          parameter when specifying any "VM" related value.

        .. note::

          Because this is an asynchronous call, it will always return 200, whether an association
          occurs or not. To confirm if an association was found based on the provided details, call
          ``ListDiscoveredResources`` .

        See also: `AWS API Documentation
        <https://docs.aws.amazon.com/goto/WebAPI/AWSMigrationHub-2017-05-31/PutResourceAttributes>`_

        **Request Syntax**
        ::

          response = client.put_resource_attributes(
              ProgressUpdateStream='string',
              MigrationTaskName='string',
              ResourceAttributeList=[
                  {
                      'Type':
                      'IPV4_ADDRESS'|'IPV6_ADDRESS'|'MAC_ADDRESS'|'FQDN'
                      |'VM_MANAGER_ID'|'VM_MANAGED_OBJECT_REFERENCE'|'VM_NAME'
                      |'VM_PATH'|'BIOS_ID'|'MOTHERBOARD_SERIAL_NUMBER',
                      'Value': 'string'
                  },
              ],
              DryRun=True|False
          )
        :type ProgressUpdateStream: string
        :param ProgressUpdateStream: **[REQUIRED]**

          The name of the ProgressUpdateStream.

        :type MigrationTaskName: string
        :param MigrationTaskName: **[REQUIRED]**

          Unique identifier that references the migration task. *Do not store personal data in this
          field.*

        :type ResourceAttributeList: list
        :param ResourceAttributeList: **[REQUIRED]**

          Information about the resource that is being migrated. This data will be used to map the
          task to a resource in the Application Discovery Service repository.

          .. note::

            Takes the object array of ``ResourceAttribute`` where the ``Type`` field is reserved for
            the following values: ``IPV4_ADDRESS | IPV6_ADDRESS | MAC_ADDRESS | FQDN
            | VM_MANAGER_ID
            | VM_MANAGED_OBJECT_REFERENCE | VM_NAME | VM_PATH | BIOS_ID |
            MOTHERBOARD_SERIAL_NUMBER`` where the identifying value can be a string up to 256
            characters.

          .. warning::

            * If any "VM" related value is set for a ``ResourceAttribute`` object, it is required
            that ``VM_MANAGER_ID`` , as a minimum, is always set. If ``VM_MANAGER_ID`` is not set,
            then all "VM" fields will be discarded and "VM" fields will not be used for matching the
            migration task to a server in Application Discovery Service repository. See the `Example
            <https://docs.aws.amazon.com/migrationhub/latest/ug/API_PutResourceAttributes.html#API_PutResourceAttributes_Examples>`__
            section below for a use case of specifying "VM" related values.

            * If a server you are trying to match has multiple IP or MAC addresses, you should
            provide as many as you know in separate type/value pairs passed to the
            ``ResourceAttributeList`` parameter to maximize the chances of matching.

          - *(dict) --*

            Attribute associated with a resource.

            Note the corresponding format required per type listed below:

              IPV4

             ``x.x.x.x``

             *where x is an integer in the range [0,255]*

              IPV6

             ``y : y : y : y : y : y : y : y``

             *where y is a hexadecimal between 0 and FFFF. [0, FFFF]*

              MAC_ADDRESS

             ``^([0-9A-Fa-f]{2}[:-]){5}([0-9A-Fa-f]{2})$``

              FQDN

             ``^[^<>{}\\\\/?,=\\p{Cntrl}]{1,256}$``

            - **Type** *(string) --* **[REQUIRED]**

              Type of resource.

            - **Value** *(string) --* **[REQUIRED]**

              Value of the resource type.

        :type DryRun: boolean
        :param DryRun:

          Optional boolean flag to indicate whether any effect should take place. Used to test if
          the caller has permission to make the call.

        :rtype: dict
        :returns:

          **Response Syntax**

          ::

            {}
          **Response Structure**

          - *(dict) --*
        """

    @overload
    # pylint: disable=arguments-differ,redefined-outer-name,redefined-builtin
    def get_paginator(
        self, operation_name: Literal["list_created_artifacts"]
    ) -> paginator_scope.ListCreatedArtifactsPaginator:
        """
        Get Paginator for `list_created_artifacts` operation.
        """

    @overload
    # pylint: disable=arguments-differ,redefined-outer-name,redefined-builtin
    def get_paginator(
        self, operation_name: Literal["list_discovered_resources"]
    ) -> paginator_scope.ListDiscoveredResourcesPaginator:
        """
        Get Paginator for `list_discovered_resources` operation.
        """

    @overload
    # pylint: disable=arguments-differ,redefined-outer-name,redefined-builtin
    def get_paginator(
        self, operation_name: Literal["list_migration_tasks"]
    ) -> paginator_scope.ListMigrationTasksPaginator:
        """
        Get Paginator for `list_migration_tasks` operation.
        """

    @overload
    # pylint: disable=arguments-differ,redefined-outer-name,redefined-builtin
    def get_paginator(
        self, operation_name: Literal["list_progress_update_streams"]
    ) -> paginator_scope.ListProgressUpdateStreamsPaginator:
        """
        Get Paginator for `list_progress_update_streams` operation.
        """

    # pylint: disable=arguments-differ,redefined-outer-name,redefined-builtin
    def get_paginator(self, operation_name: str) -> Boto3Paginator:
        """
        Create a paginator for an operation.

        :type operation_name: string
        :param operation_name: The operation name.  This is the same name
            as the method name on the client.  For example, if the
            method name is ``create_foo``, and you'd normally invoke the
            operation as ``client.create_foo(**kwargs)``, if the
            ``create_foo`` operation can be paginated, you can use the
            call ``client.get_paginator("create_foo")``.

        :raise OperationNotPageableError: Raised if the operation is not
            pageable.  You can use the ``client.can_paginate`` method to
            check if an operation is pageable.

        :rtype: L{botocore.paginate.Paginator}
        :return: A paginator object.
        """


class Exceptions:
    AccessDeniedException: Boto3ClientError
    ClientError: Boto3ClientError
    DryRunOperation: Boto3ClientError
    HomeRegionNotSetException: Boto3ClientError
    InternalServerError: Boto3ClientError
    InvalidInputException: Boto3ClientError
    PolicyErrorException: Boto3ClientError
    ResourceNotFoundException: Boto3ClientError
    ServiceUnavailableException: Boto3ClientError
    UnauthorizedOperation: Boto3ClientError
